@@ -59,7 +59,7 @@ function Grub()
 
 function InstallTools() {
       # Store terminal choice from settings
-      terminal=$(cat $2|tr -d '[]"')
+      terminal=$2
 
       applicationRun=$1
       applicationInstall=$1
@@ -74,7 +74,7 @@ function InstallTools() {
       if [ "$1" == "stacer" ]; then
             # Install Stacer if necessary
             if [[ $(which stacer | wc -l) -eq 0 ]]; then
-                  $terminal "yay -S $1 --noconfirm"
+                  eval $terminal "yay -S $1 --noconfirm"
             fi
             # Run Stacer
             $1 &
@@ -83,25 +83,25 @@ function InstallTools() {
       elif [ "$1" == "pace" ]; then
             # Install Pace if necessary
             if [[ $(which pace | wc -l) -eq 0 ]]; then
-                  $terminal "yay -S $1 --noconfirm"
+                  eval $terminal "yay -S $1 --noconfirm"
             fi
             # Run Pace
             $1 &
 
       # If pamac was the application selected, run the below code
-elif [ "$applicationInstall" == "pamac" ]; then
-            # Install Pamac if necessary
-            if [[ $(which pamac | wc -l) -eq 0 ]]; then
-                  $terminal "yay -S $applicationInstall --noconfirm"
-            fi
-            # Run Pamac
-            $applicationRun &
+      elif [ "$applicationInstall" == "pamac" ]; then
+                  # Install Pamac if necessary
+                  if [[ $(which pamac | wc -l) -eq 0 ]]; then
+                        $terminal "yay -S $applicationInstall --noconfirm"
+                  fi
+                  # Run Pamac
+                  $applicationRun &
 
-      # If pyakm was the application selected, run the below code
- elif [ "$applicationInstall" == "pyakm" ]; then
+            # If pyakm was the application selected, run the below code
+      elif [ "$applicationInstall" == "pyakm" ]; then
             # Install Pyakm if necessary
             if [[ $(which pyakm | wc -l) -eq 0 ]]; then
-                  $terminal "yay -S $applicationInstall --noconfirm"
+                  eval $terminal "yay -S $applicationInstall --noconfirm"
             fi
             # Run Pyakm
             $applicationRun &
@@ -110,89 +110,92 @@ elif [ "$applicationInstall" == "pamac" ]; then
 
 function DesktopEnvironment() {
       # Detect the user-specified terminal
-      terminal=$(cat $2|tr -d '[]"')
+      terminal=$2
 
       if [ "$1" == "gnome" ]; then
             echo "gnome"
-            $terminal "sudo pacman -S gnome-common gnome-font-viewer gnome-keyring gnome-terminal qt5-styleplugins qt5ct gnome-online-accounts libgnome-keyring --needed --noconfirm"
+            eval $terminal "sudo pacman -S gnome-common gnome-font-viewer gnome-keyring gnome-terminal qt5-styleplugins qt5ct gnome-online-accounts libgnome-keyring --needed --noconfirm"
       elif [ "$1" == "apricity" ]; then
             echo "apricity"
             # Download and install apricity-theme from the main RebornOS repository
             curl -o rebornos.txt http://repo.rebornos.org/RebornOS/
             themeFile=$(grep "apricity-theme" rebornos.txt | cut -f2 -d "\"" | cut -d "\"" -f1 | grep -v "sig")
             # Install packages with only one pkexec dialog
-            $terminal "sudo pacman -U $themeFile --needed --noconfirm"
+            eval $terminal "sudo pacman -U $themeFile --needed --noconfirm"
             rm -f rebornos.txt
       elif [ "$1" == "cinnamon" ]; then
             echo "cinnamon"
-            $terminal "sudo pacman -S $1 muffin nemo-fileroller nemo-share nemo-preview nemo-seahorse seahorse gnome-common gnome-font-viewer gnome-keyring gnome-terminal qt5-styleplugins qt5ct gnome-online-accounts libgnome-keyring --needed --noconfirm"
-            $terminal "sudo yay -S mint-themes mint-x-icons mint-y-icons mintlocale mintstick --noconfirm --needed"
+            eval $terminal "sudo pacman -S $1 muffin nemo-fileroller nemo-share nemo-preview nemo-seahorse seahorse gnome-common gnome-font-viewer gnome-keyring gnome-terminal qt5-styleplugins qt5ct gnome-online-accounts libgnome-keyring --needed --noconfirm"
+            eval $terminal "sudo yay -S mint-themes mint-x-icons mint-y-icons mintlocale mintstick --noconfirm --needed"
       elif [ "$1" == "deepin" ]; then
             echo "deepin"
-            $terminal "sudo pacman -S redshift deepin deepin-extra noto-fonts noto-fonts-emoji unicode-character-database blueman --needed --noconfirm"
-            $terminal "sudo pacman -Rdd qt5ct --noconfirm"
+            eval $terminal "sudo pacman -S redshift deepin deepin-extra noto-fonts noto-fonts-emoji unicode-character-database blueman --needed --noconfirm"
+            eval $terminal "sudo pacman -Rdd qt5ct --noconfirm"
       elif [ "$1" == "budgie" ]; then
             echo "budgie"
-            $terminal "sudo pacman -S bugdie-desktop budgie-extras gnome gnome-tweaks --needed --noconfirm"
+            eval $terminal "sudo pacman -S bugdie-desktop budgie-extras gnome gnome-tweaks --needed --noconfirm"
       elif [ "$1" == "enlightenment" ]; then
             echo "enlightenment"
-            $terminal "sudo pacman -S enlightenment connman qt5ct terminology xdg-user-dirs-gtk --needed --noconfirm"
+            eval $terminal "sudo pacman -S enlightenment connman qt5ct terminology xdg-user-dirs-gtk --needed --noconfirm"
       elif [ "$1" == "i3" ]; then
             echo "i3"
-            $terminal "sudo pacman -S i3 dmenu file-roller network-manager-applet networkmanager-openvpn networkmanager-pptp nitrogen pcmanfm polkit-gnome xdg-user-dirs-gtk notify-osd python-pysmbc lxappearance-gtk3 lxinput-gtk3 awesome-terminal-fonts --needed --noconfirm"
+            eval $terminal "sudo pacman -S i3 dmenu file-roller network-manager-applet networkmanager-openvpn networkmanager-pptp nitrogen pcmanfm polkit-gnome xdg-user-dirs-gtk notify-osd python-pysmbc lxappearance-gtk3 lxinput-gtk3 awesome-terminal-fonts --needed --noconfirm"
       elif [ "$1" == "plasma" ]; then
             echo "plasma"
-            $terminal "sudo pacman -S gnu-free-fonts spectacle kcolorchooser sshfs gwenview plasma kf5 kdebase --needed --noconfirm"
+            eval $terminal "sudo pacman -S gnu-free-fonts spectacle kcolorchooser sshfs gwenview plasma kf5 kdebase --needed --noconfirm"
       elif [ "$1" == "lxqt" ]; then
             echo "lxqt"
-            $terminal "sudo pacman -S lxqt packagekit-qt5 xdg-user-dirs libstatgrab libsysstat network-manager-applet file-roller unace xscreensaver xautolock lxqt-build-tools --needed --noconfirm"
+            eval $terminal "sudo pacman -S lxqt packagekit-qt5 xdg-user-dirs libstatgrab libsysstat network-manager-applet file-roller unace xscreensaver xautolock lxqt-build-tools --needed --noconfirm"
       elif [ "$1" == "mate" ]; then
             echo "mate"
-            $terminal "sudo pacman -S mate mate-extra gnome-keyring network-manager-applet networkmanager-applet xdg-user-dirs-gtk mate-tweak qt5ct"
+            eval $terminal "sudo pacman -S mate mate-extra gnome-keyring network-manager-applet networkmanager-applet xdg-user-dirs-gtk mate-tweak qt5ct"
       elif [ "$1" == "openbox" ]; then
             echo "openbox"
-            $terminal "sudo pacman -S lxde file-roller arandr openbox-menu compton lxmenu-data lxappearance-obconf-gtk3 obkey oblogout networkmanager-openvpn networkmanager-pptp notify-osd nitrogen mousepad obconf parcellite polkit qt5ct scrot tint2 volumeicon udiskie xdg-user-dirs-gtk openbox-theme --needed --noconfirm"
+            eval $terminal "sudo pacman -S lxde file-roller arandr openbox-menu compton lxmenu-data lxappearance-obconf-gtk3 obkey oblogout networkmanager-openvpn networkmanager-pptp notify-osd nitrogen mousepad obconf parcellite polkit qt5ct scrot tint2 volumeicon udiskie xdg-user-dirs-gtk openbox-theme --needed --noconfirm"
       elif [ "$1" == "xfce" ]; then
             echo "xfce"
-            $terminal "sudo pacman -S xfce4 xfce4-goodies file-roller gtk-engine-murrine network-manager-applet networkmanager-openvpn networkmanager-pptp polkit-gnome qt5ct xdg-user-dirs-gtk ristretto xfce4-whiskermenu-plugin gnome-font-viewer catfish libgit2-glib --needed --noconfirm"
+            eval $terminal "sudo pacman -S xfce4 xfce4-goodies file-roller gtk-engine-murrine network-manager-applet networkmanager-openvpn networkmanager-pptp polkit-gnome qt5ct xdg-user-dirs-gtk ristretto xfce4-whiskermenu-plugin gnome-font-viewer catfish libgit2-glib --needed --noconfirm"
       elif [ "$1" == "pantheon" ]; then
             echo "pantheon"
-            $terminal "yay -S pantheon-session-git gala-git wingpanel-indicator-datetime-git wingpanel-indicator-session-git wingpanel-git cerbere-git pantheon-applications-menu-git switchboard-git pantheon-dpms-helper-git pantheon-default-settings-git gnome-settings-daemon-elementary pantheon-print-git pantheon-polkit-agent-git --noconfirm"
+            eval $terminal "yay -S pantheon-session-git gala-git wingpanel-indicator-datetime-git wingpanel-indicator-session-git wingpanel-git cerbere-git pantheon-applications-menu-git switchboard-git pantheon-dpms-helper-git pantheon-default-settings-git gnome-settings-daemon-elementary pantheon-print-git pantheon-polkit-agent-git --noconfirm"
             pkexec pacman -S pantheon bzr --needed --noconfirm
       fi
 }
 
-function Rollback()
-{
-      #Declare variables
-      line=$1
-      file=$2
+function Rollback() {
+      # Declare variables
+      day=$1
+      month=$2
+      year=$3
       # Detect the user-specified terminal
-      terminal=$(cat $2|tr -d '[]"')
-      # Set values for the indicated line
-      month=$(sed -n "$line p" < $file | awk '{print $1}')
-      day=$(sed -n "$line p" < $1file | awk '{print $2}')
-      year=$(sed -n "$line p" < $file | awk '{print $3}')
-      cp /etc/pacman.d/mirrorlist /etc/pacman.d/mirrorlist.bak
-      echo "Server=https://archive.archlinux.org/repos/$year/$month/$day/$repo/os/$arch" > /etc/pacman.d/mirrorlist
-      $terminal "sudo pacman -Syyuu --noconfirm"
+      terminal=$4
+
+      # Create the correct URL variable for the mirrorlist file
+      url1="Server = https://archive.archlinux.org/repos/$year/$month/$day/"
+      url2='$repo/os/$arch'
+      url=$url1$url2
+      # Backup the up-to-date mirrorlist
+      if [ $(grep -w '/etc/pacman.d/mirrorlist' -e 'Server = https://archive.archlinux.org' | wc -l) -eq 0 ]; then
+            cp /etc/pacman.d/mirrorlist /etc/pacman.d/mirrorlist.bak
+      fi
+      echo $url > /etc/pacman.d/mirrorlist
+      eval $terminal "sudo pacman -Syyuu --noconfirm"
 }
 
 function ResetRollback()
 {
       # Store terminal choice from settings
-      terminal=$(cat $2|tr -d '[]"')
+      terminal=$1
       if [ -f /etc/pacman.d/mirrorlist.bak ]; then
             pkexec mv -f /etc/pacman.d/mirrorlist.bak /etc/pacman.d/mirrorlist
       fi
-      $terminal "sudo pacman -Syyuu --noconfirm"
+      eval $terminal "sudo pacman -Syyuu --noconfirm"
 }
 
 function Downgrade()
 {
       # Store terminal choice from settings
-      terminal=$(cat $2|tr -d '[]"')
-      $terminal "pkexec downgrade $2"
+      eval $2 "pkexec downgrade $1"
 }
 
 function Contribution()
@@ -238,8 +241,6 @@ function InstallAnbox()
 
             # Notify the user that this will most likely take a few minutes
             sudo -u ${RUID} DBUS_SESSION_BUS_ADDRESS="unix:path=/run/user/${RUSER_UID}/bus" notify-send  "Please be patient..." "This could take a few minutes." -a "RebornOS FIRE" -u critical -i "/usr/share/icons/default/reborn-updates.svg"
-            # Uninstall kernel headers for Anbox
-            sudo pacman -Rns anbox-modules-dkms-git
             # Uninstall the Anbox snap package
             sudo snap remove anbox
             # Remove SystemD service
@@ -270,24 +271,9 @@ function InstallAnbox()
                   echo "Snap is already installed!"
             fi
             if [ -f /etc/pacman.d/reborn-mirrorlist ]; then
-                  sudo pacman -S anbox-modules-dkms-git snapd lzip curl squashfs-tools wget tar unzip --noconfirm --needed
+                  sudo pacman -S snapd lzip curl squashfs-tools wget tar unzip --noconfirm --needed
             else
                   sudo pacman -S snapd lzip curl squashfs-tools wget tar unzip --noconfirm --needed
-
-                  # Install anbox-modules-dkms-git from the AUR
-                  yay -G --aur anbox-modules-dkms-git
-                  # Set permissions so that makepkg can run without root priviledges
-                  chown -R 777 anbox-git
-                  chmod 777 anbox-git
-                  cd anbox-git
-                  # Install makedepends, as makepkg would prompt for them otherwise
-                  sudo pacman -S $(grep "makedepends" PKGBUILD | tr -d "=()'" | cut -c 12-) --needed --noconfirm
-                  # Use makepkg to create the package locally
-                  sudo -u ${RUID} makepkg --noconfirm --needed --rmdeps
-                  # Install the created package
-                  sudo pacman -U $(ls | grep "anbox-modules-dkms-git") --noconfirm --needed
-                  cd ..
-                  sudo rm -rf anbox-git
             fi
             # If snap was not installed until the previous command...
             if [ "$snapInstalled" == "false" ]; then
@@ -429,7 +415,7 @@ function ReceiveWormhole()
 BuildMycroft()
 {
       # Detect the user-specified terminal
-      terminal=$(cat $1|tr -d '[]"')
+      terminal=$1
       # Get the Real Username
       RUID=$(who | awk 'FNR == 1 {print $1}')
       # Translate Real Username to Real User ID
@@ -444,7 +430,7 @@ BuildMycroft()
       fi
       sudo -u ${RUID} DBUS_SESSION_BUS_ADDRESS="unix:path=/run/user/${RUSER_UID}/bus" git clone https://github.com/MycroftAI/mycroft-core.git
       cd mycroft-core
-      sudo -u ${RUID} DBUS_SESSION_BUS_ADDRESS="unix:path=/run/user/${RUSER_UID}/bus" $terminal bash $USER_HOME/mycroft-core/dev_setup.sh
+      sudo -u ${RUID} DBUS_SESSION_BUS_ADDRESS="unix:path=/run/user/${RUSER_UID}/bus" eval $terminal bash $USER_HOME/mycroft-core/dev_setup.sh
 }
 
 StartMycroft()
@@ -520,7 +506,7 @@ TextMycroft()
 RemoveMycroft()
 {
       # Detect the user-specified terminal
-      terminal=$(cat $1|tr -d '[]"')
+      terminal=$1
       # Get the Real Username
       RUID=$(who | awk 'FNR == 1 {print $1}')
       # Translate Real Username to Real User ID
@@ -531,12 +517,28 @@ RemoveMycroft()
 
       sudo rm /usr/share/applications/mycroft.desktop
       if [ -d $USER_HOME/mycroft-core ]; then
-            sudo -u ${RUID} DBUS_SESSION_BUS_ADDRESS="unix:path=/run/user/${RUSER_UID}/bus" $terminal bash $USER_HOME/mycroft-core/dev_setup.sh --clean
+            sudo -u ${RUID} DBUS_SESSION_BUS_ADDRESS="unix:path=/run/user/${RUSER_UID}/bus" eval $terminal bash $USER_HOME/mycroft-core/dev_setup.sh --clean
             sudo rm -rf $USER_HOME/mycroft-core
       fi
 }
 
-export -f LightDM SDDM TestLightdm SDDMdefault LightDMdefault Grub InstallTools DesktopEnvironment Rollback ResetRollback Downgrade Contribution InstallAnbox SetupWormhole SendWormhole ReceiveWormhole BuildMycroft StartMycroft StopMycroft TextMycroft RemoveMycroft
+CleaningOptions()
+{
+      # If $1 == 1
+      if [[ $1 == "1" ]]; then
+            yes | pacman -Scc
+      fi
+      # If $2 == 1
+      if [[ $2 == "1" ]]; then
+            journalctl --vacuum-time=3d && sync
+      fi
+      # If $3 == 1
+      if [[ $3 == "1" ]]; then
+            reflector --verbose -p https --sort rate --save /etc/pacman.d/mirrorlist
+      fi
+}
+
+export -f LightDM SDDM TestLightdm SDDMdefault LightDMdefault Grub InstallTools DesktopEnvironment Rollback ResetRollback Downgrade Contribution InstallAnbox SetupWormhole SendWormhole ReceiveWormhole BuildMycroft StartMycroft StopMycroft TextMycroft RemoveMycroft CleaningOptions
 export PATH=$PATH:/var/lib/snapd/snap/bin
 
 "$@"
