@@ -34,6 +34,8 @@ settingsFile = os.path.abspath(os.path.join(os.path.dirname( __file__ ), '..', '
 
 # Create Handlers (Triggers) for each item
 class Handler:
+    with open(settingsFile) as outfile:
+        self.settings = json.load(outfile)
 
 # Close the window
     def onDestroy(self, *args):
@@ -53,7 +55,7 @@ class Handler:
 
 # Build / Update Mycroft
     def onBuild(self, button):
-        os.system("pkexec " + bashFile + " BuildMycroft " + settingsFile)
+        os.system("pkexec " + bashFile + " BuildMycroft " + self.settings["terminal"])
 
 # Type queries for Mycroft
     def onEnter(self, pkgtxt):
