@@ -12,6 +12,7 @@ import gi
 import os
 import time
 import json
+from datetime import date
 from pathlib import Path
 gi.require_version('Gtk', '3.0')
 from gi.repository import Gtk
@@ -46,7 +47,6 @@ installPantheon = 'bash ' + workingDirectory + '/reborn-updates Pantheon'
 os.system('bash ' + startupFile + ' FirstTime ' + runFile + ' ' + packageFile + ' ' + SettingsFile + ' ' + workingDirectory)
 os.system('bash ' + startupFile + ' DisplayLightDMgreeters ' + workingDirectory)
 os.system('bash ' + startupFile + ' DisplaySDDMgreeters ' + workingDirectory)
-os.system('bash ' + startupFile + ' StorePacmanLog ' + workingDirectory)
 
 # Check for Internet connection
 conn = httplib.HTTPConnection("www.google.com", timeout=5)
@@ -57,11 +57,151 @@ except:
     conn.close()
     Notify.Notification.new("Lacking Internet connection. Some options may not work").show()
 
+# Create function for calculating unnecessary storage space
+def TotalStorage():
+    # Declare variables
+    packageStorage = 0.0
+    storageUnit = "K"
+    # Store the space that Pacman's cache is taking up in kilobytes
+    cacheStorage = subprocess.check_output("du -k /var/cache/pacman/pkg/ | awk '{print $1}'", shell=True)
+    cacheStorage = float(cacheStorage.decode())
+    # Store the space that the logs are taking up in kilobytes
+    if os.path.exists("/run/log/journal"):
+        journalStorage = subprocess.check_output("du -k /run/log/journal/ | awk '{print $1}' | sed -n '1p'", shell=True)
+    else:
+        journalStorage = subprocess.check_output("du -k /var/log/journal/ | awk '{print $1}' | sed -n '1p'", shell=True)
+    if journalStorage.decode() == '':
+        journalStorage = 0
+    else:
+        journalStorage = float(journalStorage.decode())
+
+    # Iterate over every line in package-files.txt to calculate the sum of all the storage that the unnecessary packages are currently taking up
+    os.system('pacman -Qdtq >' + workingDirectory + '/package-files.txt')
+    with open(workingDirectory + '/package-files.txt', 'r') as packageFile:
+        Lines = packageFile.readlines()
+        for line in Lines:
+            # Store the size of the package
+            packageStorageTemp = subprocess.check_output("pacman -Qi " + line.strip() + " | grep 'Installed Size' | awk '{print $4}'", shell=True)
+            packageStorageTemp = float(packageStorageTemp.strip().decode())
+            # Store the unit that the size of the package is displayed in
+            packageSize = subprocess.check_output("pacman -Qi " + line.strip() + " | grep 'Installed Size' | awk '{print $5}'", shell=True)
+            packageSize = str(packageSize.strip().decode())
+            # If the package size is displayed in MiB, convert it to KiB
+            if packageSize == "MiB":
+                packageStorageTemp = packageStorageTemp * 1024
+            # If the package size is displayed in GiB, convert to KiB
+            elif packageSize == "GiB":
+                packageStorageTemp = packageStorageTemp * 1048576
+            # Add the current package's size to the running total for the file
+            packageStorage += packageStorageTemp
+
+    # Add the cache, journal, and package sizes together for one number
+    storageSaved = cacheStorage + journalStorage + packageStorage
+
+    # If amount is at least as large as a gigabyte
+    if storageSaved > 7812500:
+        storageSaved = storageSaved / 7812500
+        storageUnit = "GB"
+    # If amount is at least as large as a Megabyte
+    elif storageSaved > 7812.5:
+        storageSaved = storageSaved / 7812.5
+        storageUnit = "MB"
+    # Round to 2 decimal places
+    storageSaved = round(storageSaved, 2)
+
+    # Return the amount saved along with its corrosponding unit
+    return " " +  str(storageSaved) + " " + storageUnit
+
+# Function that calculates current journal space
+def JournalStorage():
+    # Store the space that the logs are taking up in kilobytes
+    if os.path.exists("/run/log/journal"):
+        journalStorage = subprocess.check_output("du -k /run/log/journal/ | awk '{print $1}' | sed -n '1p'", shell=True)
+    else:
+        journalStorage = subprocess.check_output("du -k /var/log/journal/ | awk '{print $1}' | sed -n '1p'", shell=True)
+
+    if journalStorage.decode() == '':
+        journalStorage = 0
+    else:
+        journalStorage = float(journalStorage.decode())
+        journalStorage = round(journalStorage, 2)
+    return " (" + str(journalStorage) + " K)"
+
+# Function that calculates current unnecessary cache space
+def CacheStorage():
+    # Store the space that Pacman's cache is taking up in kilobytes
+    cacheStorage = subprocess.check_output("du -k /var/cache/pacman/pkg/ | awk '{print $1}'", shell=True)
+    cacheStorage = float(cacheStorage.decode())
+
+    storageUnit = "KB"
+    # If amount is at least as large as a gigabyte
+    if cacheStorage > 7812500:
+        cacheStorage = cacheStorage / 7812500
+        storageUnit = "GB"
+    # If amount is at least as large as a Megabyte
+    elif cacheStorage > 7812.5:
+        cacheStorage = cacheStorage / 7812.5
+        storageUnit = "MB"
+
+    cacheStorage = round(cacheStorage, 2)
+    return " (" + str(cacheStorage) + " " + storageUnit + ")"
+
+# Function that calculates current unnecessary cache space
+def PackageStorage():
+    packageStorage = 0.0
+    # Iterate over every line in package-files.txt to calculate the sum of all the storage that the unnecessary packages are currently taking up
+    os.system('pacman -Qdtq >' + workingDirectory + '/package-files.txt')
+    with open(workingDirectory + '/package-files.txt', 'r') as packageFile:
+        Lines = packageFile.readlines()
+        for line in Lines:
+            # Store the size of the package
+            packageStorageTemp = subprocess.check_output("pacman -Qi " + line.strip() + " | grep 'Installed Size' | awk '{print $4}'", shell=True)
+            packageStorageTemp = float(packageStorageTemp.strip().decode())
+            # Store the unit that the size of the package is displayed in
+            packageSize = subprocess.check_output("pacman -Qi " + line.strip() + " | grep 'Installed Size' | awk '{print $5}'", shell=True)
+            packageSize = str(packageSize.strip().decode())
+            # If the package size is displayed in MiB, convert it to KiB
+            if packageSize == "MiB":
+                packageStorageTemp = packageStorageTemp * 1024
+            # If the package size is displayed in GiB, convert to KiB
+            elif packageSize == "GiB":
+                packageStorageTemp = packageStorageTemp * 1048576
+            # Add the current package's size to the running total for the file
+            packageStorage += packageStorageTemp
+
+    storageUnit = "KB"
+    # If amount is at least as large as a gigabyte
+    if packageStorage > 7812500:
+        packageStorage = packageStorage / 7812500
+        storageUnit = "GB"
+    # If amount is at least as large as a Megabyte
+    elif packageStorage > 7812.5:
+        packageStorage = packageStorage / 7812.5
+        storageUnit = "MB"
+
+    packageStorage = round(packageStorage, 2)
+    return " (" + str(packageStorage) + " " + storageUnit + ")"
+
 # Create Handlers (Triggers) for each item
 class Handler:
     def __init__(self):
-        self.listoptions1 = [0,0,0,0,0]
+        self.listoptions1 = [0,0,0,0]
         self.listoptions2 = [0,0,0,0,0]
+        # Create settings file if it does not already exist, as well as declare the self.settings array
+        if not os.path.isfile(settingsFile):
+            self.settings = {
+                "terminal": "",
+                "chosen_date": {
+                    "day": 0,
+                    "month": 0,
+                    "year": 0
+                }
+            }
+            with open(settingsFile, 'w+') as outfile:
+                outfile.write(json.dumps(self.settings, indent = 4))
+        else:
+            with open(settingsFile) as outfile:
+                self.settings = json.load(outfile)
 
 # Close the window
     def onDestroy(self, *args):
@@ -141,62 +281,71 @@ class Handler:
 
     # Install Budgie
     def clickBudgie(self, button):
-        os.system('bash ' + bashFile + ' DesktopEnvironment budgie ' + settingsFile)
+        os.system('bash ' + bashFile + ' DesktopEnvironment budgie ' + self.settings["terminal"])
 
     # Install Cinnamon
     def clickCinnamon(self, button):
-        os.system('bash ' + bashFile + ' DesktopEnvironment cinnamon ' + settingsFile)
+        os.system('bash ' + bashFile + ' DesktopEnvironment cinnamon ' + self.settings["terminal"])
 
     # Install Deepin
     def clickDeepin(self, button):
-        os.system('bash ' + bashFile + ' DesktopEnvironment deepin ' + settingsFile)
+        os.system('bash ' + bashFile + ' DesktopEnvironment deepin ' + self.settings["terminal"])
 
     # Install Enlightenment
     def clickEnlightenment(self, button):
-        os.system('bash ' + bashFile + ' DesktopEnvironment enlightenment ' + settingsFile)
+        os.system('bash ' + bashFile + ' DesktopEnvironment enlightenment ' + self.settings["terminal"])
 
     # Install GNOME
     def clickGnome(self, button):
-        os.system('bash ' + bashFile + ' DesktopEnvironment gnome ' + settingsFile)
+        os.system('bash ' + bashFile + ' DesktopEnvironment gnome ' + self.settings["terminal"])
 
     # Install i3
     def clicki3(self, button):
-        os.system('bash ' + bashFile + ' DesktopEnvironment i3 ' + settingsFile)
+        os.system('bash ' + bashFile + ' DesktopEnvironment i3 ' + self.settings["terminal"])
 
     # Install Plasma
     def clickPlasma(self, button):
-        os.system('bash ' + bashFile + ' DesktopEnvironment plasma ' + settingsFile)
+        os.system('bash ' + bashFile + ' DesktopEnvironment plasma ' + self.settings["terminal"])
 
     # Install LXQt
     def clickLXQt(self, button):
-        os.system('bash ' + bashFile + ' DesktopEnvironment lxqt ' + settingsFile)
+        os.system('bash ' + bashFile + ' DesktopEnvironment lxqt ' + self.settings["terminal"])
 
     # Install Mate
     def clickMate(self, button):
-        os.system('bash ' + bashFile + ' DesktopEnvironment mate ' + settingsFile)
+        os.system('bash ' + bashFile + ' DesktopEnvironment mate ' + self.settings["terminal"])
 
     # Install Openbox
     def clickOpenbox(self, button):
-        os.system('bash ' + bashFile + ' DesktopEnvironment openbox ' + settingsFile)
+        os.system('bash ' + bashFile + ' DesktopEnvironment openbox ' + self.settings["terminal"])
 
     # Install Pantheon
     def clickPantheon(self, button):
-        os.system('bash ' + bashFile + ' DesktopEnvironment pantheon ' + settingsFile)
+        os.system('bash ' + bashFile + ' DesktopEnvironment pantheon ' + self.settings["terminal"])
 
     # Install XFCE
     def clickXFCE(self, button):
-        os.system('bash ' + bashFile + ' DesktopEnvironment xfce ' + settingsFile)
+        os.system('bash ' + bashFile + ' DesktopEnvironment xfce ' + self.settings["terminal"])
 
 ################################################################################
 ############################### Third Tab ######################################
 ################################################################################
 
+    # Select the general option
+    def onRemoveAll(self, switch,state):
+        if state == True:
+            # Set all the below switches to "True"
+            builder.get_object("goClearCache").set_active("True")
+            builder.get_object("goCleanJourna").set_active("True")
+            builder.get_object("goUnnecessaryPrograms").set_active("True")
+
     # Clear Pacman's cache completely
     def onClearCache(self, switch, state):
         if state == True:
             self.listoptions1[0]=1
         else:
             self.listoptions1[0]=0
+            builder.get_object("goRemoveAll").set_active(False)
 
     # Clean the system journal excepting the last 3 days worth of text
     def onCleanJournal(self, switch, state):
@@ -204,13 +353,15 @@ class Handler:
             self.listoptions1[1]=1
         else:
             self.listoptions1[1]=0
+            builder.get_object("goRemoveAll").set_active(False)
 
-    # Clean package list
-    def onCleanPackages(self, switch, state):
+    # Allow the user to remove programs that are no longer necessary - orphans, depreciated, etc
+    def onUnnecessaryPrograms(self, switch, state):
         if state == True:
             self.listoptions1[2]=1
         else:
             self.listoptions1[2]=0
+            builder.get_object("goRemoveAll").set_active(False)
 
     # Rank the mirrors
     def onRankMirrors(self, switch, state):
@@ -219,42 +370,35 @@ class Handler:
         else:
             self.listoptions1[3]=0
 
-    # Allow the user to remove programs that are no longer necessary - orphans, depreciated, etc
-    def onUnnecessaryPrograms(self, switch, state):
-        if state == True:
-            self.listoptions1[4]=1
-        else:
-            self.listoptions1[4]=0
-
     # Apply select changes from above
     def clickApply1(self, button):
         print(self.listoptions1)
         if self.listoptions1[0] == 1:
-            print("Clearing Cache (leaving nothing behind)...")
-            os.system('yes| pkexec pacman -Scc')
-            Notify.Notification.new("Cache Cleared (leaving nothing behind)").show()
+            print("Clearing Cache...")
 
         if self.listoptions1[1] == 1:
             print("Cleaning Journal...")
-            os.system('pkexec journalctl --vacuum-time=3d && sync')
-            Notify.Notification.new("Journal Cleaned").show()
-
-        if self.listoptions1[2] == 1:
-            print("Cleaning Package Cache (leaving the 3 most recent package versions)...")
-            os.system('pkexec paccache -ruk0')
-            Notify.Notification.new("Package Cache Cleaned (leaving the 3 most recent package versions)").show()
 
         if self.listoptions1[3] == 1:
             print("Ranking Mirrors...")
             Notify.Notification.new("Ranking Mirrors...").show()
-            os.system('pkexec reflector --verbose -p https --sort rate --save /etc/pacman.d/mirrorlist')
-            Notify.Notification.new("Mirrors Ranked").show()
 
-        if self.listoptions1[4] == 1:
-            Notify.Notification.new("Generating Package List...").show()
-            os.system('rm -f ' + workingDirectory + '/package-files.txt')
-            os.system('pkexec pacman -Qdtq >' + workingDirectory + '/package-files.txt')
-            os.system('python3 ' + UnnecessaryFile)
+        # Clear cache, clean journal, and rank mirrors
+        if self.listoptions1[0] == 1 or self.listoptions1[1] == 1 or self.listoptions1[3] == 1:
+            os.system( "pkexec " + bashFile + " CleaningOptions " + str(self.listoptions1[0]) + ' ' + str(self.listoptions1[1]) +  ' ' + str(self.listoptions1[3]))
+
+        # Run Python3 command
+        if self.listoptions1[2] == 1:
+            # If the file containing the list of packages that can be removed is empty, show the below command.
+            if os.stat(workingDirectory + "/package-files.txt").st_size == 0:
+                Notify.Notification.new("No Unnecessary Packages to Remove").show()
+            # Otherwise, show the dialog window to remove those unnecessary packages
+            else:
+                os.system("python3 " + UnnecessaryFile)
+
+        # Display "Done" message
+        if self.listoptions1[0] == 1 or self.listoptions1[1] == 1 or self.listoptions1[2] == 1 or self.listoptions1[3] == 1:
+            Notify.Notification.new("Done!").show()
 
 ################################################################################
 ############################### Fourth Tab #####################################
@@ -322,14 +466,55 @@ class Handler:
 ############################### Fifth Tab ######################################
 ################################################################################
 
-    def rollbackUse(self, pkgtxt):
-        global enteredText
-        enteredText = pkgtxt.get_text()
-        os.system('pkexec bash ' + bashFile + ' Rollback ' + enteredText + ' ' + workingDirectory + '/transactions.txt ' + settingsFile)
+    def setRollbackDate(self, selectedDate):
+        # Record the full date that has been selected by the user
+        rollbackDate = builder.get_object("RollbackCalendar").get_date()
+        rollbackDate = " ".join([str(i) for i in rollbackDate])
+        # Extract the year from the selected date
+        rollbackYear = rollbackDate.split(' ')[0]
+
+        # Extract the month from the selected date. By default, it is numerically 1 lower in value than it should be.
+        rollbackMonth = str(int(rollbackDate.split(' ')[1]) + 1)
+        # If the month is not a double-digit value, we must manually tack on a leading 0
+        if int(rollbackMonth) < 10:
+            rollbackMonth = '0' + rollbackMonth
+
+        # Extract the day from the selected date
+        rollbackDay = rollbackDate.split(' ')[2]
+        # If the day is not a double-digit value, we must manually tack on a leading 0
+        if int(rollbackDay) < 10:
+            rollbackDay = '0' + rollbackDay
+
+        # Since we have modified the month and day values, just use our 3 values as an updated rollbackDate value
+        rollbackDate = rollbackYear + rollbackMonth + rollbackDay
+
+        # If the date selected is in the future, prevent the user from using said date
+        if int(rollbackDate.replace(" ", "")) > int(str(date.today()).replace("-", "")):
+            Notify.Notification.new("Cannot Rollback to the Future").show()
+            builder.get_object("RollbackCalendar").set_property("day", date.today().day)
+            builder.get_object("RollbackCalendar").set_property("month", date.today().month - 1)
+            builder.get_object("RollbackCalendar").set_property("year", date.today().year)
+        # If the date selected is before Arch Linux began archiving their packages, replace the selected date with the earliest possible
+        elif int(rollbackDate.replace(" ", "")) < 20130831:
+            Notify.Notification.new("Archives did NOT exist before August 31, 2013").show()
+            builder.get_object("RollbackCalendar").set_property("day", 31)
+            builder.get_object("RollbackCalendar").set_property("month", 7)
+            builder.get_object("RollbackCalendar").set_property("year", 2013)
+        # If neither of the two above cases were true, then save the date in the settings file
+        else:
+            self.settings["chosen_date"]["day"] = int(rollbackDay)
+            self.settings["chosen_date"]["month"] = int(rollbackMonth)
+            self.settings["chosen_date"]["year"] = int(rollbackYear)
+            # Update values in JSON settings file
+            with open(settingsFile, 'w+') as outfile:
+                outfile.write(json.dumps(self.settings, indent = 4))
+
+    def rollbackUse(self, button):
+        os.system('pkexec bash ' + bashFile + ' Rollback ' + self.settings["chosen_date"]["day"] + ' ' + self.settings["chosen_date"]["month"] + ' ' + self.settings["chosen_date"]["year"] + ' ' + self.settings["terminal"])
         Notify.Notification.new("Rollback Complete. Please Reboot for Full Effect").show()
 
     def resetRollbackUse(self, button):
-        os.system('bash ' + bashFile + ' ResetRollback ' + settingsFile)
+        os.system('bash ' + bashFile + ' ResetRollback ' + self.settings["terminal"])
 
 ################################################################################
 ############################### Sixth Tab ######################################
@@ -340,7 +525,7 @@ class Handler:
         # Check to see if Mycroft/s desktop file already is already in place
         mycroftFile = Path("/usr/share/applications/mycroft.desktop")
         if mycroftFile.exists():
-            os.system('pkexec bash ' + bashFile + ' RemoveMycroft ' + settingsFile)
+            os.system('pkexec bash ' + bashFile + ' RemoveMycroft ' + self.settings["terminal"])
             # Inform the user by updating the button label
             builder.get_object("Mycroft").set_label(locale["RebornOSFIRE"]["ProgramsTab"]["AddonsTab"]["ControlMycroft"])
         else:
@@ -378,19 +563,19 @@ class Handler:
 
     # Open Pace
     def clickPacman(self, button):
-        os.system('bash ' + bashFile + ' InstallTools pace ' + settingsFile)
+        os.system('bash ' + bashFile + ' InstallTools pace ' + self.settings["terminal"])
 
     # Open Arch Kernel Manager
     def clickKernel(self, button):
-        os.system('bash ' + bashFile + ' InstallTools pyakm-manager ' + settingsFile)
+        os.system('bash ' + bashFile + ' InstallTools pyakm-manager ' + self.settings["terminal"])
 
     # Open Stacer
     def clickStacer(self, button):
-        os.system('bash ' + bashFile + ' InstallTools stacer ' + settingsFile)
+        os.system('bash ' + bashFile + ' InstallTools stacer ' + self.settings["terminal"])
 
     # Open Pamac
     def clickPamac(self, button):
-        os.system('bash ' + bashFile + ' InstallTools pamac-manager ' + settingsFile)
+        os.system('bash ' + bashFile + ' InstallTools pamac-manager ' + self.settings["terminal"])
 
 ################################################################################
 ############################### Drawing App Window #############################
@@ -420,14 +605,6 @@ with open(workingDirectory + '/Settings/sddm.txt', 'r') as sddmFile:
         builder.get_object("SDDMUseThemeBox").append_text(line.strip())
 # =====================================
 
-# =====================================
-# View the file when the window pops up
-# =====================================
-with open(workingDirectory + '/Settings/transactions.txt', 'r') as rollbackFile:
-    rollbackData = rollbackFile.read()
-    builder.get_object("rollbackView").get_buffer().set_text(rollbackData)
-# =====================================
-
 # =====================================
 # Set text through the use of keys from the JSON files used for translations
 # =====================================
@@ -472,11 +649,18 @@ with open(localeDirectory + '/translations_' + os.getenv('LANG').split('_')[0] +
     main_stack.child_set_property(system_tasks_page, "title", locale["RebornOSFIRE"]["SystemTasksTab"]["SystemTasks"])
     # -----> System Maintenance sub-tab
     builder.get_object("SystemMaintenanceTab").set_text(locale["RebornOSFIRE"]["SystemTasksTab"]["SystemMaintenanceTab"]["SystemMaintenance"])
-    builder.get_object("CacheRemove").set_text(locale["RebornOSFIRE"]["SystemTasksTab"]["SystemMaintenanceTab"]["CacheRemove"])
-    builder.get_object("CacheLeave").set_text(locale["RebornOSFIRE"]["SystemTasksTab"]["SystemMaintenanceTab"]["CacheLeave"])
-    builder.get_object("CleanJournal").set_text(locale["RebornOSFIRE"]["SystemTasksTab"]["SystemMaintenanceTab"]["CleanJournal"])
+    # Calculate the space that can be saved and add it on to the end of the text
+    removeAll = locale["RebornOSFIRE"]["SystemTasksTab"]["SystemMaintenanceTab"]["RemoveAll"] + TotalStorage()
+    builder.get_object("RemoveAll").set_text(removeAll)
+    builder.get_object("RemoveAdvanced").set_text(locale["RebornOSFIRE"]["SystemTasksTab"]["SystemMaintenanceTab"]["RemoveAdvanced"])
+    removeCache = locale["RebornOSFIRE"]["SystemTasksTab"]["SystemMaintenanceTab"]["CacheRemove"] + CacheStorage()
+    builder.get_object("CacheRemove").set_text(removeCache)
+    removeJournal = locale["RebornOSFIRE"]["SystemTasksTab"]["SystemMaintenanceTab"]["CleanJournal"] + JournalStorage()
+    builder.get_object("CleanJournal").set_text(removeJournal)
+    removePackages = locale["RebornOSFIRE"]["SystemTasksTab"]["SystemMaintenanceTab"]["RemoveUnnecessaryPrograms"] + PackageStorage()
+    builder.get_object("RemoveUnnecessaryPrograms").set_text(removePackages)
+
     builder.get_object("RankMirrors").set_text(locale["RebornOSFIRE"]["SystemTasksTab"]["SystemMaintenanceTab"]["RankMirrors"])
-    builder.get_object("RemoveUnnecessaryPrograms").set_text(locale["RebornOSFIRE"]["SystemTasksTab"]["SystemMaintenanceTab"]["RemoveUnnecessaryPrograms"])
     builder.get_object("goApply3").set_label(locale["RebornOSFIRE"]["SystemTasksTab"]["SystemMaintenanceTab"]["Apply"])
     # -----> Repair sub-tab
     builder.get_object("RepairTab").set_text(locale["RebornOSFIRE"]["SystemTasksTab"]["RepairTab"]["Repair"])
@@ -531,8 +715,13 @@ with open(localeDirectory + '/translations_' + os.getenv('LANG').split('_')[0] +
     main_stack.child_set_property(rollback_page, "title", locale["RebornOSFIRE"]["RollbackTab"]["Rollback"])
 
     builder.get_object("RollbackSystem").set_text(locale["RebornOSFIRE"]["RollbackTab"]["RollbackSystem"])
-    builder.get_object("InsertOptionNumber").set_placeholder_text(locale["RebornOSFIRE"]["RollbackTab"]["InsertOptionNumber"])
+    builder.get_object("RollbackUse").set_label(locale["RebornOSFIRE"]["RollbackTab"]["Rollback"])
     builder.get_object("UpgradeToCurrent").set_label(locale["RebornOSFIRE"]["RollbackTab"]["UpgradeToCurrent"])
+
+    # Ensure calendar is set to the current date
+    builder.get_object("RollbackCalendar").set_property("day", date.today().day)
+    builder.get_object("RollbackCalendar").set_property("month", date.today().month - 1)
+    builder.get_object("RollbackCalendar").set_property("year", date.today().year)
 # =====================================
 
 window2 = builder.get_object("RebornOSFIRE")
